@@ -4,11 +4,13 @@ from enum import IntEnum
 from typing import Sequence, Optional
 
 import sqlalchemy as sa
+from pure_sql import pure_sql
 from pydantic import BaseModel
 from sqlalchemy import create_engine, ForeignKey
 from sqlalchemy.engine import Engine, Connection
 
 # from twbm.environment import Environment
+from vimania.environment import ROOT_DIR
 
 _log = logging.getLogger("vimania-plugin.dal")
 logging.getLogger("sqlalchemy.engine").setLevel(logging.DEBUG)
@@ -86,46 +88,8 @@ class DAL:
         """returns the minimal flag of all children,
         None if no children, or not found
         """
-        query = """
-            with recursive tds_children as (
-                select id,
-                       parent_id,
-                       todo,
-                       metadata,
-                       tags,
-                       desc,
-                       path,
-                       flags,
-                       last_update_ts,
-                       created_at,
-                       0 as depth
-                from vimania_todos
-                where id == :id_
-                  and flags <= 4
-                union all
-                select vimania_todos.id,
-                       vimania_todos.parent_id,
-                       vimania_todos.todo,
-                       vimania_todos.metadata,
-                       vimania_todos.tags,
-                       vimania_todos.desc,
-                       vimania_todos.path,
-                       vimania_todos.flags,
-                       vimania_todos.last_update_ts,
-                       vimania_todos.created_at,
-                       tds_children.depth + 1
-                from vimania_todos
-                         join tds_children
-                where vimania_todos.parent_id == tds_children.id
-                  and vimania_todos.flags <= 4
-            )
-            select min(flags) overall_status
-            from tds_children
-            where id != :id_
-            --where level < 3  -- only one level down
-            ;
-        """
-        result = self.conn.execute(query, id_=id_).all()
+        q = pure_sql.get_query_by_name("get_overall_status", f"{ROOT_DIR}/db/sql/hierarchy.sql")
+        result = self.conn.execute(q.sql, id_=id_).all()
         _log.debug(f"{id_=}, {result=}")
         self.conn.connection.commit()
         try:
@@ -135,26 +99,8 @@ class DAL:
             return None
 
     def get_depth(self, id_: int) -> int:
-        query = """
-            with recursive tds_parents as (
-                select id, parent_id, todo, 0 as depth
-                from vimania_todos
-                where id == :id_
-                  and flags <= 4
-                union all
-                select vimania_todos.id, vimania_todos.parent_id, vimania_todos.todo, tds_parents.depth - 1
-                from vimania_todos
-                         join tds_parents
-                where vimania_todos.id == tds_parents.parent_id
-                  and vimania_todos.flags < 4
-            )
-            select depth
-            from tds_parents
-            order by depth
-            limit 1
-            ;
-        """
-        result = self.conn.execute(query, id_=id_).all()
+        q = pure_sql.get_query_by_name("get_depth", f"{ROOT_DIR}/db/sql/hierarchy.sql")
+        result = self.conn.execute(q.sql, id_=id_).all()
         _log.debug(f"{id_=}, {result=}")
         self.conn.connection.commit()
         try:
@@ -163,45 +109,8 @@ class DAL:
             return 0
 
     def get_todo_parent(self, id_: int, depth: int) -> Todo:
-        query = """
-            with recursive tds_parents as (
-                select id,
-                       parent_id,
-                       todo,
-                       metadata,
-                       tags,
-                       desc,
-                       path,
-                       flags,
-                       last_update_ts,
-                       created_at,
-                       0 as depth
-                from vimania_todos
-                where id = :id_
-                  and flags <= 4
-                union all
-                select vimania_todos.id,
-                       vimania_todos.parent_id,
-                       vimania_todos.todo,
-                       vimania_todos.metadata,
-                       vimania_todos.tags,
-                       vimania_todos.desc,
-                       vimania_todos.path,
-                       vimania_todos.flags,
-                       vimania_todos.last_update_ts,
-                       vimania_todos.created_at,
-                       tds_parents.depth - 1
-                from vimania_todos
-                         join tds_parents
-                where vimania_todos.id == tds_parents.parent_id
-                  and vimania_todos.flags <= 4
-            )
-            select *
-            from tds_parents
-            where depth = :depth
-            ;
-        """
-        result = self.conn.execute(query, id_=id_, depth=depth).all()
+        q = pure_sql.get_query_by_name("get_todo_parent", f"{ROOT_DIR}/db/sql/hierarchy.sql")
+        result = self.conn.execute(q.sql, id_=id_, depth=depth).all()
         assert len(result) == 1, f"Ambigouus parents: {result=}, {id_=}"
         self.conn.connection.commit()
         return result[0]
@@ -226,8 +135,10 @@ class DAL:
             values (:parent_id, :todo, :metadata, :tags, :desc, :path, :flags, :created_at)
             returning *;
         """
+        q = pure_sql.get_query_by_name("insert_todo", f"{ROOT_DIR}/db/sql/crud.sql")
         result = self.conn.execute(
-            query,
+            q.sql,
+            # query,
             parent_id=todo.parent_id,
             todo=todo.todo,
             metadata=todo.metadata,
@@ -236,9 +147,9 @@ class DAL:
             path=todo.path,
             flags=todo.flags,
             created_at=datetime.utcnow(),
-        ).all()
+        ).lastrowid
         self.conn.connection.commit()
-        return result[0][0]
+        return result
 
     def update_todo(self, todo: Todo) -> int:
         query = """
@@ -249,6 +160,7 @@ class DAL:
             returning *;
         """
         result = self.conn.execute(
+            # q.sql,
             query,
             id=todo.id,
             parent_id=todo.parent_id,
@@ -265,16 +177,8 @@ class DAL:
         return result[0][0]
 
     def get_todo_by_id(self, id_: int) -> Todo:
-        # Example query
-        # noinspection SqlResolve
-        query = """
-            -- name: get_todo_by_id^
-            -- record_class: Todo
-            select *
-            from vimania_todos
-            where id = :id;
-            """
-        sql_result = self.conn.execute(query, id=id_).all()
+        q = pure_sql.get_query_by_name("get_todo_by_id", f"{ROOT_DIR}/db/sql/crud.sql")
+        sql_result = self.conn.execute(q.sql, id=id_).all()
         if not sql_result:
             # noinspection PyRedundantParentheses
             return Todo()
@@ -310,49 +214,15 @@ class DAL:
 
     def get_related_tags(self, tag: str):
         tag_query = f"%,{tag},%"
-        # noinspection SqlResolve
-        query = """
-            -- name: get_related_tags
-            with RECURSIVE split(tags, rest) AS (
-                SELECT '', tags || ','
-                FROM vimania_todos
-                WHERE tags LIKE :tag_query
-                -- WHERE tags LIKE '%,ccc,%'
-                UNION ALL
-                SELECT substr(rest, 0, instr(rest, ',')),
-                       substr(rest, instr(rest, ',') + 1)
-                FROM split
-                WHERE rest <> '')
-            SELECT distinct tags
-            FROM split
-            WHERE tags <> ''
-            ORDER BY tags;
-        """
+        q = pure_sql.get_query_by_name("get_related_tags", f"{ROOT_DIR}/db/sql/crud.sql")
         sql_result = self.conn.execute(
-            query,
+            q.sql,
             tag_query=tag_query,
         ).all()
         return [tags[0] for tags in sql_result]
 
     def get_all_tags(self):
         # noinspection SqlResolve
-        query = """
-            -- name: get_all_tags
-            with RECURSIVE split(tags, rest) AS (
-                SELECT '', tags || ','
-                FROM vimania_todos
-                UNION ALL
-                SELECT substr(rest, 0, instr(rest, ',')),
-                       substr(rest, instr(rest, ',') + 1)
-                FROM split
-                WHERE rest <> '')
-            SELECT distinct tags
-            FROM split
-            WHERE tags <> ''
-            ORDER BY tags;
-        """
-        sql_result = self.conn.execute(
-            query,
-        ).all()
-
+        q = pure_sql.get_query_by_name("get_all_tags", f"{ROOT_DIR}/db/sql/crud.sql")
+        sql_result = self.conn.execute(q.sql).all()
         return [tags[0] for tags in sql_result]
